function [XB, flag, iter, re, time] = Attr_Main_Block(A, B, node_label1, node_label2, l_node, X0, H, tol, k1, max_iter, alpha)
%% main function for the FASTEN-N solver
% ################################################################################################## %
%--INPUTS:
%   A, B: adjacency matrix of two graphs
%   node_label1, node_label2: node label vectors
%   l_node: # of node labels
%   X0: the initial solution for the equation, usually zero matrix
%   H: the preference matrix; If no prior anchor link is known, set it as
%   uniform matrix
%   tol: tolerance of the solver, usually a small scalar like 1e-6
%   k1: subspace size of A, B respectively, usually a small scalar in [2,60]
%   max_iter: max iteration number
%   alpha: parameter of the Sylvester equation; in (0,1)
%--OUTPUTS:
%   XB: solution
%   flag: 1: converge 0: not converge
%   iter: iteration number
%   re: residual value
%   time: running time
% ##################################################################################################### %


m = size(A,1);
n = size(B,1);

Da = sum(A, 2).^(-0.5);
Db = sum(B, 2).^(-0.5);
Da(Da==Inf) = 0;
Db(Db==Inf) = 0;
Acel = cell(1, l_node);
A2cel = cell(1,l_node);
B2cel = cell(1,l_node);
Bcel = cell(1, l_node);
Xt = cell(1,l_node);
Bt = cell(1,l_node);
l = @Moperator_Block;
for i=1:l_node
    [rn1,cn1,~] = find(node_label1 == i);
    [rn2,cn2,~] = find(node_label2 == i);        
    N1i = sparse(rn1, cn1, 1, m, 1);
    N2i = sparse(rn2, cn2, 1, n, 1);
    for j = 1:l_node
        
        [rn3,cn3,~] = find(node_label1 == j);
        [rn4,cn4,~] = find(node_label2 == j);
        
        N1j = sparse(rn3, cn3, 1, m, 1);
        N2j = sparse(rn4, cn4, 1, n, 1);
        
        if i == j
            Bt{j} = bsxfun(@times, bsxfun(@times, N2j, ones(n,m)/(m*n)), N1j');
            Xt{j} = sparse(n,m);
        end

        An = bsxfun(@times, bsxfun(@times, N1i, A), N1j');
        Bn = bsxfun(@times, bsxfun(@times, N2i, B), N2j');
 
        Aj = alpha^(0.5)*bsxfun(@times, bsxfun(@times, Da, An), Da');
        Bj = alpha^(0.5)*bsxfun(@times, bsxfun(@times, Db, Bn), Db');
        Aj2 = Aj;
        Bj2 = Bj;
        [r1i, ~] = find(N1i==0);
        [~, c1j] = find(N1j'==0);
        [r2i, ~] = find(N2i==0);
        [~, c2j] = find(N2j'==0);
      
        Aj2(r1i, :) =[];
        Aj2(:,c1j) = [];
   
        Bj2(r2i, :) =[];
        Bj2(:,c2j) = [];

        Acel{i, j} = Aj;
        Bcel{i, j} = Bj;
        A2cel{i,j} = Aj2;
        B2cel{i,j} = Bj2;
%     end
    end
end

tic;
Xsum = X0;
Re0 = H - zeros(n,m) + l(Acel, Bcel, Xt, l_node);
re = norm(Re0, 'fro');
iter = 0;flag = 0;
while iter < max_iter
    iter = iter +1;
    fprintf('iteration %d ',iter);
    if re < tol
        XB = Xsum;
        flag = 1;
        break;
    else
        for t = 1 : l_node
                if t == 1
                    Bsum = Bt{t};
                else
                    Bsum = Bsum + Bt{t};            
                end
        end
        
        for i = 1:l_node
                Bj = Bt{i} + Operator2_Block(Acel, Bcel, Xt, l_node, i);
%             for j = 1:l_node
                
%                 if i == j 
                    % call main4Block
                    [X1, ~, it1, ~, tim] = main4Block(Acel{i, i}, Bcel{i, i}, sparse(n,m), Bj, tol, k1, k1, 1e-10, 30, 100);
                    Xt{i} = -X1;
                    fprintf('iter1:%d ',it1);
                    fprintf('time:%d ',tim);
%                 end
%                 if i == j && i ~= 1
% %                     % call main4Block
%                     [X1, ~, it2] = main4Block(Acel{j, j}, Bcel{j, j}, sparse(m,n), Bj, tol, k2, k2, 1e-10, 30, 100);
%                     Xt{i} = -X1;
%                     fprintf('iter2:%d \n',it2);
%                     if norm(Bsum-Xsum-X1+l(Acel, Bcel, Xt, l_node), 'fro') < norm(Bsum-Xsum-Xt{i}+l(Acel, Bcel, Xt, l_node), 'fro')
%                         Xt{i} = X1;
%                     end
%                 end
%                 if i ~= j && j == 1
%                     % call main2nd4Block
%                     R = -(Bt{j} + Operator2_Block(Acel, Bcel, Xt, l_node, i)) + Xt{j};
%                     [r2, ~] = find(node_label2 == j);
%                     [r1, ~] = find(node_label1 == j);
%                     R(r2, :) = [];
%                     R(:, r1') = [];
%                     X00 = sparse(size(B2cel{j,i},2),size(A2cel{j,i}',1));
%                     [~, X1, ~, ~,~] = main2nd4Block(A2cel{j, i}, B2cel{j, i}, X00, R, tol, 5, 5, 1e-10, 30, 100);
% %                     X1 = linsolve(full(kron(Acel{j, i}, Bcel{j, i})), full(reshape(-(Bt{j} + Operator2_Block(Acel, Bcel, Xt, l_node, i))+ Xt{j},m*n,1)));
% %                     X1 = reshape(X1, m,n);
%                     Xt{i} = X1;
%                 end
%                 if i ~= j && j ~= 1
%                     % call main2nd4Block
%                     R = -(Bt{j} + Operator2_Block(Acel, Bcel, Xt, l_node, i)) + Xt{j};
%                     [r2, ~] = find(node_label2 ~= j);
%                     [r1, ~] = find(node_label1 ~= j);
%                     R(r2, :) = [];
%                     R(:, r1') = [];
%                     X00 = sparse(size(B2cel{j,i},2),size(A2cel{j,i}',1));
% %                     [~, X1, ~, ~,~] = main2nd4Block(A2cel{j, i}, B2cel{j, i}, X00, R , tol, 5, 5, 1e-10, 30, 100);
%                     X1 = linsolve(full(kron(A2cel{j, i}, B2cel{j, i})), full(reshape(R,size(R,1)*size(R,2),1)));
%                     X1 = reshape(X1, size(R,1),size(R,2));
%                     if norm(Bsum-Xsum-X1+l(Acel, Bcel, Xt, l_node), 'fro') < norm(Bsum-Xsum-Xt{i}+l(Acel, Bcel, Xt, l_node), 'fro')
%                         Xt{i} = X1;
%                     end
%                 end
%             end
%                Xsum = Xsum + Xt{i};
            % find smallest Xj, let Xt{i} = Xj
        end
        for t = 1 : l_node
            if t == 1
        %         Bsum = Bt{i};
                Xsum = Xt{t};
            else
        %         Bsum = Bsum + Bt{i};
                Xsum = Xsum + Xt{t};
            end
        end
        Xsum(Xsum==0) = H(1,1); % this is when R is an unifrom matrix, should be changed if not
        % update Re0 
        Re0 = H - Xsum + l(Acel, Bcel, Xt, l_node);
        re = norm(Re0, 'fro');
        fprintf('residual: %d\n', re);
    end
end
XB = Xsum;
time=toc;
end


