function [X, flag, iter, time] = main(A, Aq, X0, R, tol, m, n, epsilon, max_iter, CGmax_iter, alpha)
%% main function of Sylvester Equation method
% ################################################################################################## %
%--INPUTS:
%   A, Aq: adjacency matrix of two graphs
%   X0: the initial solution for the equation, usually zero matrix
%   R: the preference matrix; If no prior anchor link is known, set it as
%   uniform matrix
%   tol: tolerance of the solver, usually a small scalar like 1e-6
%   m, n: subspace size of A, Aq respectively, usually a small scalar in [2,60]
%   epsilon: the tolerance of the Glocal CG method used in the solver,
%   usually smaller than tol (e.g. 1e-7)
%   max_iter: max iteration number
%   CGmax_iter: max iteration number for Global CG method
%   alpha: parameter of the Sylvester equation; in (0,1)
%--OUTPUTS:
%   X: solution
%   flag: 1: converge 0: not converge
%   iter: iteration number
%   time: running time
% ##################################################################################################### %

da = sum(A,2);
db = sum(Aq, 2);
n1 = size(A, 1);
n2 = size(Aq, 1);
% A = alpha^(0.5).*bsxfun(@times, bsxfun(@times, da.^(-0.5), A),
% (da.^(-0.5))');% not efficient
% Aq = alpha^(0.5).*bsxfun(@times, bsxfun(@times, db.^(-0.5), Aq),
% (db.^(-0.5))'); % not efficient
D1 = sparse(1:n1, 1:n1, da.^(-0.5));
D2 = sparse(1:n2, 1:n2, db.^(-0.5));
A(isnan(A))=0;
Aq(isnan(Aq))=0;
A = alpha^(0.5).*D1*A*D1;
Aq = alpha^(0.5).*D2*Aq*D2;

tic;
Re0 = R - Aq*X0*A + X0;
iter = 0;flag = 0;
while iter < max_iter
    if toc > 3000
        break
    end
        
    iter = iter +1;
    if norm(Re0,'fro') < tol
        X = X0;
%         re = norm(Re0,'fro');
        flag = 1;
        break
    else
        start = tic;
        [f,g] = subspace(Re0);
        [V,Ha] = arnoldi(A, g, n);
        [W,Hb] = arnoldi(Aq, f, m);
        Arnotime = toc(start);
        fprintf('Arnotime:%d ',Arnotime);
        
        CGstart = tic;
        E = Hb'*W'*Re0*V*Ha - W(:,1:m)'*Re0*V(:,1:n);
        [Y, flag, CGiter, ~] =  GLCG(sparse(zeros(m,n)), E, epsilon, CGmax_iter, Ha, Hb, m, n);
        CGtime = toc(CGstart);
        fprintf('CGtime:%d ',CGtime);
        fprintf('CGiter:%d ',CGiter);
        
        X1 = X0 + W(:, 1:m)*Y*V(:, 1:n)';
        Re1 = Re0 - W*Hb*Y*Ha'*V' + W(:,1:m)*Y*V(:,1:n)';
        re = norm(Re1, 'fro');
        X0 = X1;
        Re0 = Re1;
        fprintf('out_iter:%d\n ', iter);
        fprintf('residual:%d\n ',re);
    end
end
X = X0;
time = toc;
end
